import os
from pathlib import Path
import platform


def get_default_minecraft_home(edition: str = "java") -> Path:
    """
    Returns the default Minecraft directory based on OS and edition.

    edition: "java" (default) or "bedrock"
    """
    home = Path.home()
    system = platform.system()

    if edition.lower() == "java":
        if system == "Windows":
            return Path(os.getenv("APPDATA", home / "AppData/Roaming")) / ".minecraft"
        elif system == "Darwin":  # macOS
            return home / "Library" / "Application Support" / "minecraft"
        else:  # Linux, BSD, etc.
            return home / ".minecraft"

    elif edition.lower() == "bedrock":
        if system == "Windows":
            local_appdata = Path(os.getenv("LOCALAPPDATA", home / "AppData/Local"))
            return (
                local_appdata
                / "Packages"
                / "Microsoft.MinecraftUWP_8wekyb3d8bbwe"
                / "LocalState"
                / "games"
                / "com.mojang"
            )
        elif system == "Darwin":  # macOS (iOS not supported here)
            return (
                home
                / "Library"
                / "Containers"
                / "com.mojang.minecraftpe"
                / "Data"
                / "Documents"
                / "games"
                / "com.mojang"
            )
        else:  # Linux/Android → not standardized
            return Path("/sdcard/games/com.mojang")

    else:
        raise ValueError(f"Unknown edition: {edition!r}")


def check_if_valid_home(home: Path) -> bool:
    required_subdirs = ["versions", "assets", "saves"]

    for subdir in required_subdirs:
        dir = home / subdir
        if not dir.is_dir():
            return False

    return True


def check_if_valid_save(home: Path) -> bool:
    required_subdirs = ["playerdata", "region"]
    required_files = ["level.dat"]

    for subdir in required_subdirs:
        dir = home / subdir
        if not dir.is_dir():
            return False

    for file_name in required_files:
        dir = home / file_name
        if not dir.is_file():
            return False

    return True



from pathlib import Path
import shutil


def make_backup(save_dir: Path, world_name: str) -> str:
    output_filename = save_dir / f"{world_name}-before_restore"
    dir_name = save_dir / world_name
    return shutil.make_archive(output_filename, "zip", dir_name)



from pathlib import Path
from typing import Annotated
import typer

from hardcore_restore.backup import make_backup
from hardcore_restore.restore import restore
from hardcore_restore.utils import (
    check_if_valid_home,
    check_if_valid_save,
    get_default_minecraft_home,
)
from hardcore_restore.statistics import nerf_death_count


default_home: Path = get_default_minecraft_home()


def run(
    world_name: Annotated[str, typer.Argument()],
    minecraft_home: Annotated[
        Path, typer.Option("--minecraft-home", "-m")
    ] = default_home,
    do_backup: bool = True,
    change_stats: bool = False,
    move_x: Annotated[float, typer.Option("--move-x", "-rx")] = 0,
    move_y: Annotated[float, typer.Option("--move-y", "-ry")] = 0,
    move_z: Annotated[float, typer.Option("--move-z", "-rz")] = 0,
):

    if not check_if_valid_home(minecraft_home):
        # typer.echo("Minecraft Home not exists, try to specify with -m")
        while not check_if_valid_home(minecraft_home):
            typer.echo("Minecraft Home dir doesn't seem to be valid!")
            minecraft_home = typer.prompt(
                "Please provide the correct minecraft home! (leave empty for default)",
                default=default_home,
                type=Path,
            )

    world_dir = minecraft_home / "saves" / world_name

    if not world_dir.is_dir():
        typer.echo(
            f"Error, no world exists in {world_dir} (no directory). Check if it is correct",
            err=True,
        )
        raise typer.Exit(1)

    if not check_if_valid_save(world_dir):
        typer.echo(
            f"Error, the given world doesn't seem like a valid minecraft world in {world_dir}. Check if it is correct",
            err=True,
        )
        raise typer.Exit(1)

    if do_backup:
        typer.echo("Making backup..")
        backup_zip = make_backup(minecraft_home / "saves", world_name)
        typer.echo(
            typer.style(f"backup is under {backup_zip}", bold=True)
            + ", simply unzip to recover!"
        )

    typer.echo(f"Changing world in {world_dir}!")
    restore(world_dir, (move_x, move_y, move_z))

    if change_stats:
        typer.echo(f"Changing stats..")
        nerf_death_count(world_dir)


def main():
    typer.run(run)


if __name__ == "__main__":
    main()



from pathlib import Path
from typing import Tuple
import nbtlib as nbt
from nbtlib.tag import Short, Float, Int, Double


def restore(world_dir: Path, move: Tuple[float, float, float]):
    with nbt.load(world_dir / "level.dat") as level:

        level["Data"]["Player"]["foodLevel"] = Int(20)
        level["Data"]["Player"]["Health"] = Float(10)
        if level["Data"]["Player"].get("LastDeathLocation") is not None:
            del level["Data"]["Player"]["LastDeathLocation"]
        level["Data"]["Player"]["DeathTime"] = Short(0)

        level["Data"]["Player"]["playerGameType"] = Int(0)

        for cord_index in range(3):
            level["Data"]["Player"]["Pos"][cord_index] += Double(move[cord_index])

    playerdata_dir = world_dir / "playerdata"
    for file in playerdata_dir.glob("*.dat"):
        with nbt.load(file) as f:

            f["foodLevel"] = Int(20)
            f["Health"] = Float(10)
            if f.get("LastDeathLocation") is not None:
                del f["LastDeathLocation"]
            f["DeathTime"] = Short(0)

            f["playerGameType"] = Int(0)

            for cord_index in range(3):
                f["Pos"][cord_index] += Double(move[cord_index])



import json
from pathlib import Path


def nerf_death_count(world_dir: Path):
    stats_dir = world_dir / "stats"
    for stat_file in stats_dir.glob("*.json"):
        with open(stat_file, "r") as f:
            data = json.load(f)

            data["stats"]["minecraft:custom"]["minecraft:deaths"] = 0
            data["stats"]["minecraft:custom"]["minecraft:time_since_death"] = 0

        with open(stat_file, "w") as f:
            json.dump(data, f)
